/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */

#ifndef _GEOMETRY_H_
#define _GEOMETRY_H_

/** geometry.h */
EXTERN struct t_geometry {

	/** true if geometry is plane parallel, set in comment */
	bool lgGeoPP;

	/** angle of illumination for incident continuum in radians - 
	 * default is normal illumination, or 0 */
	realnum AngleIllumRadian;

	/** direction cosin for incident continuum entering AngleIllumRadian
	 * from normal - this is 1./cos(normal), so > 1. */
	realnum DirectionalCosin;

	/** FillFac is filling factor, default 1,
	* others are parameters to make function of radius */
	realnum FillFac, 
	  filpow, 
	  fiscal;

	/** flag saying whether spherical geometry */
	bool lgSphere;

	/** default open geometry, covgeo is unity, covrt is zero.  
	 * closed geometry, covgeo is unity, and covrt is unity */

	/* covering factors, account for possible less than total coverageof 4\pi */ 
	/** linearly affects line luminosities but not intensities (since they are
	 * per unit area of cloud.
	 * When sphere command is entered with no optional covering factor this is
	 * set to 1 and covrt is set to covgeo.  The default is an open geometry,
	 * and covgeo is unity for this as well */
	realnum covgeo;

	/** radiative transfer covering factor accounts for diffuse radiation
	 * escaping from system in open geometry.  weakly affects intensities and
	 * luminosities.  For open geometry this is zero (half of diffuse fields
	 * escape from cloud and do not strike gas, for a closed geometry this is unity 
	 * also set to covgeo (def unity) if covering factor command is entered */
	realnum covrt;

	/** flag saying that spherical geometry is static, set with sphere static */
	bool lgStatic;

	/** option to simulate a slit projected onto the nebula.  This is false by
	 * default, and set true with SLIT option on the SPHERE command 
	bool lgSlit;*/

	/** this is power that goes with lgSlit - default is 0 when sphere slit
	 * is set, but is 1 if long keyword appears 
	 * default set in zerologic is 2 */
	long int iEmissPower;

	/** flag saying that it is ok to not iterate when sphere static is set,
	 * set with (OK) option on sphere static, used for testing hydrogen atom */
	bool lgStaticNoIt;

	/** nprint is how many zones to print */
	long int  nprint;

	/** nend[iteration] is limiting number of zones for that iteration */
	long int *nend /**<[ITR DIM]*/;

	/* the largest number of zones needed in any iteration, used to malloc
	 * arrays that save source function */
	long int nend_max;

	/** lgZonSet set if stopping zone specified */
	bool lgZoneSet;

	/** lgZoneTrp set if stopped due to zone number */
	bool lgZoneTrp;

	/** default limit to number of zones */
	long int nEndDflt;

	/** lgEndDflt true if still at default */
	bool lgEndDflt;

	}	geometry;



#endif /* _GEOMETRY_H_ */
