/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*InitCoreload one time initialization of core load, called from cdDrive, this sets
* minimum set of values needed for the code to start - called after
* input lines have been read in and checked for VARY or GRID - so
* known whether single or multiple sims will be run */
#include "cddefines.h"
#include "optimize.h"
#include "parse.h"
#include "dense.h"
#include "hcmap.h"
#include "h2.h"
#include "version.h"
#include "hextra.h"
#include "mole.h"
#include "extinc.h"
#include "heavy.h"
#include "grid.h"
#include "ionbal.h"
#include "iso.h"
#include "taulines.h"
/*  */
/* following used to enter total predicted continuum into emission line array 
* actually entered into line array in lineset1
*/
#include "predcont.h"
/* energies where diffuse continuum is to be entered into line array 
* variables are declared in predcont.h, where array dimensioned 
* NB - if these numbers change, the wavelength in the printout will change 
* too, since the wavelength is derived form this */
/* >>>chng 99 mar 23, adjusted energies so that wavelength line list is
* the same as it was in C90 - small changes were caused by going over
* to proper Rydberg constant */
realnum EnrPredCont[NPREDCONT] = {
	/* this is radio continuum at 3.4 cm */
	  2.680e-06f
	, 7.445e-04f 
	, 1.498e-03f 
	, 2.211e-03f 
	, 2.952e-03f 
	, 3.677e-03f 
	, 3.7501e-03f /* Ney-Allen */
	, 3.9915e-03f /* Ney-Allen */
	, 4.2543e-03f /* Ney-Allen */
	, 4.314e-03f 
	, 4.6446e-03f /* Ney-Allen */
	, 5.162e-03f 
	, 5.2462e-03f /* Ney-Allen */
	, 5.8079e-03f /* Ney-Allen */
	, 6.240e-03f 
	/* 	, 7.320e-03  */
	, 7.3312e-03f /* Ney-Allen */
	, 7.9936e-03f /* Ney-Allen */
	, 8.7119e-03f /* Ney-Allen */
	, 9.6125e-03f /* Ney-Allen */
	, 9.77243e-03f
	, 1.1099e-02f /* Ney-Allen */
	, 1.2022e-02f /* Ney-Allen */
	, 1.29253e-02f 
	, 2.2152e-02f 
	, 3.92044e-02f 
	, 5.54593e-02f 
	/* next two either side of n=4 edge of hydrogen, set to 1.5% off either direction*/
	/* >>chng 00 sep 18, had been too close in energy */
	, 6.1563e-02f 
	, 6.3437e-02f 
	, 8.1460e-02f 
	/* >>chng 00 sep 14, changed energies of paschen jump to be farther away as
	* per note on BJ */
	, 0.1094f 
	, 0.1128f 
	, 0.14675f 
	, 0.18653f 
	/* >>chng 00 sep 14, next two energies changed since they were too close to BJ
	* and so both ended up shortward of limit*/
	, 0.246f   /* these two are the Balmer jump, below and above. */
	, 0.254f  /* continuum binning not much better than 1% so offset energies by more */
	, 0.375f  /* peak on two photon continuum */
	, 0.38096f 
	, 0.43994f 
	, 0.44394f 
	, 0.50811f 
	, 0.57489f
	, 0.62487f 
	, 0.67155f 
	, 0.70244f 
	, 0.72163f 
	, 0.74812f 
	, 0.76172f 
	, 0.77551f 
	, 0.79681f 
	, 0.81859f 
	, 0.8260f 
	, 0.84859f 
	, 0.85618f 
	, 0.87967f 
	, 0.911267f /*exactly 1000A */ 
	/* points on either side of Lyman jump,
	* energies changed to be robust when energy grid changes,
	* grid resolution is about 1%, so change from 0.99783 and 1.000
	* to 1 +/- 1.5%
	* >>chng 00 sep 23 change wavelength points for next two */
	, 0.985f 
	, 1.015f 
	, 1.199f 
	, 1.299f 
	, 1.4984f 
	, 1.58441f 
	/* points on either side of Lyman jump,
	* energies changed to be robust when energy grid changes,
	* grid resolution is about 1%, so change from 1.80433 and 1.809
	* to 1.807 +/- 1.5%
	* >>chng 00 sep 23 change wavelength points for next two */
	, 1.780f 
	, 1.834f 
	, 2.283f
};

long int ipPredCont[NPREDCONT];

/*InitCoreload one time initialization of core load, called from cdDrive, this sets
* minimum set of values needed for the code to start - called after
* input lines have been read in and checked for VARY or GRID - so
* known whether single or multiple sims will be run */
void InitCoreload( void )
{
	static int nCalled=0;
	long int nelem;

	DEBUG_ENTRY( "InitCoreload()" );

	/* sanity check */
	if( nCalled )
	{
		/* return since already called */
		return;
	}
	++nCalled;

	/* number of simulation sin this coreload, 
	 * 0 while in this routine, 1 for first sim, increments just after
	 * this routine is called */
	nSimThisCoreload = 0;

	strncpy( chOptimFileName , "optimal.in" , sizeof( chOptimFileName ) );

	/* the constant that multiplies the column density to get optical depth at 1 Ryd */
	extinc.cnst_col2optdepth = 6.22e-18f;
	/* the power on the energy */
	extinc.cnst_power = -2.43f;

	hcmap.lgMapOK = true;
	hcmap.lgMapDone = false;

	/* will be reset to positive number when map actually done */
	hcmap.nMapAlloc = 0;
	hcmap.nmap = 0;
	hcmap.lgMapBeingDone = false;

	/* name of output file from optimization run */
	strncpy( chOptimFileName , "optimal.in" , sizeof( chOptimFileName ) );

	/* number of electrons in valence shell that can Compton recoil ionize */
	long int nCom[LIMELM] = 
	{
		1 , 2 ,								/* K 1s shell */
		1 , 2 ,								/* L 2s shell */
		1 , 2 , 3 , 4 , 5 , 6 ,				/* L 2p shell */
		1 , 2 ,								/* M 3s shell */
		1 , 2 , 3 , 4 , 5 , 6 ,				/* M 3p shell */
		1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ,		/* M 3d shell */
		1 , 2 ,								/* N 4s shell */
		1 , 2								/* N 4p shell */
	};

	for( nelem=ipHYDROGEN; nelem<LIMELM; ++nelem )
	{
		ionbal.nCompRecoilElec[nelem] = nCom[nelem];
	}

	/* list of shells, 1 to 7 */
	strcpy( Heavy.chShell[0], "1s" );
	strcpy( Heavy.chShell[1], "2s" );
	strcpy( Heavy.chShell[2], "2p" );
	strcpy( Heavy.chShell[3], "3s" );
	strcpy( Heavy.chShell[4], "3p" );
	strcpy( Heavy.chShell[5], "3d" );
	strcpy( Heavy.chShell[6], "4s" );

	/* variables for H-like sequence */
	/* default number of levels for hydrogen iso sequence */
	for( nelem=ipHYDROGEN; nelem < LIMELM; ++nelem )
	{
		iso.n_HighestResolved_max[ipH_LIKE][nelem] = 5;
		iso.nCollapsed_max[ipH_LIKE][nelem] = 2;
	}

	iso.n_HighestResolved_max[ipH_LIKE][ipHYDROGEN] = 10;
	iso.n_HighestResolved_max[ipH_LIKE][ipHELIUM] = 10;

	iso.nCollapsed_max[ipH_LIKE][ipHYDROGEN] = 15;
	iso.nCollapsed_max[ipH_LIKE][ipHELIUM] = 15;

	/* variables for He-like sequence */
	/* "he-like" hydrogen (H-) is treated elsewhere */
	iso.n_HighestResolved_max[ipHE_LIKE][ipHYDROGEN] = -LONG_MAX;
	iso.numLevels_max[ipHE_LIKE][ipHYDROGEN] = -LONG_MAX;
	iso.numPrintLevels[ipHE_LIKE][ipHYDROGEN] = -LONG_MAX;
	iso.nCollapsed_max[ipHE_LIKE][ipHYDROGEN] = -LONG_MAX;

	for( nelem=ipHELIUM; nelem < LIMELM; ++nelem )
	{
		/* put at least three resolved and 1 collapsed in every element for he-like */
		iso.n_HighestResolved_max[ipHE_LIKE][nelem] = 3;
		iso.nCollapsed_max[ipHE_LIKE][nelem] = 1;
	}

	iso.n_HighestResolved_max[ipHE_LIKE][ipHELIUM] = 6;
	iso.nCollapsed_max[ipHE_LIKE][ipHELIUM] = 20;

	/* And n=5 for these because they are most abundant */
	iso.n_HighestResolved_max[ipHE_LIKE][ipCARBON] = 5;
	iso.n_HighestResolved_max[ipHE_LIKE][ipNITROGEN] = 5;
	iso.n_HighestResolved_max[ipHE_LIKE][ipOXYGEN] = 5;
	iso.n_HighestResolved_max[ipHE_LIKE][ipNEON] = 5;
	iso.n_HighestResolved_max[ipHE_LIKE][ipSILICON] = 5;
	iso.n_HighestResolved_max[ipHE_LIKE][ipMAGNESIUM] = 5;
	iso.n_HighestResolved_max[ipHE_LIKE][ipSULPHUR] = 5;
	iso.n_HighestResolved_max[ipHE_LIKE][ipIRON] = 5;
	/* also set this, for exercising any possible issues with biggest charge models */
	iso.n_HighestResolved_max[ipHE_LIKE][LIMELM-1] = 5;

	iso.chISO[ipH_LIKE] = "H-like ";
	iso.chISO[ipHE_LIKE] = "He-like";

	max_num_levels = 0;
	for( long ipISO = ipH_LIKE; ipISO < NISO; ipISO++ )
	{
		for( nelem=ipISO; nelem < LIMELM; nelem++ )
		{
			/* set this to LONG_MAX, reduce to actual number later,
			 * then verify number of levels is not increased after initial coreload */
			iso.numLevels_malloc[ipISO][nelem] = LONG_MAX;
			iso_update_num_levels( ipISO, nelem );
		}
	}

	statesAdded = 0;
	lgStatesAdded = false;
	linesAdded = 0;
	lgLinesAdded = false;

	/** \todo	3	these should be const since cannot change, are flags */
	/* these are used to set trace levels of output by options on atom h2 trace command 
	 * first is minimum level of trace, keyword is FINAL */
	mole.nH2_trace_final = 1;
	/* intermediate level of trace, info per iteration, key ITERATION */
	mole.nH2_trace_iterations = 2;
	/* full trace, keyword is FULL */
	mole.nH2_trace_full = 3;
	/* print matrices used in solving X */
	mole.nH2_trace_matrix = 4;

	/* turn element on if this is first call to this routine,
	* but do not reset otherwise since would clobber how elements were set up */
	for( nelem=0; nelem < LIMELM; nelem++ )
	{
		/* never turn on element if turned off on first iteration */
		dense.lgElmtOn[nelem] = true;

		/* option to set ionization with element ioniz cmnd,
		* default (false) is to solve for ionization */
		dense.lgSetIoniz[nelem] = false;
	}

	/* smallest density to permit in any ion - if smaller then set to zero */
	dense.density_low_limit = SMALLFLOAT * 1e3;

	/* default cosmic ray background */
	/* >>chng 99 jun 24, slight change to value
	* quoted by 
	* >>refer	cosmic ray	ionization rate	McKee, C.M., 1999, astro-ph 9901370
	* this will produce a total
	* secondary ionization rate of 2.5e-17 s^-1, as tested in 
	* test suite cosmicray.in.  If each ionization produces 2.4 eV of heat,
	* the background heating rate should be 9.6e-29 * n*/
	/* >>chng 04 jan 26, update cosmic ray ionization rate for H0 to
	* >>refer	cosmic ray	ionization	Williams, J.P., Bergin, E.A., Caselli, P., 
	* >>refercon	Myers, P.C., & Plume, R. 1998, ApJ, 503, 689,
	* H0 ionization rate of 2.5e-17 s-1 and a H2 ionization rate twice this
	* >>chng 04 mar 15, comment said 2.5e-17 which is correct, but code produce 8e-17,
	* fix back to correct value 
	*/
	/* NB - the rate is derived from the density.  these two are related by the secondary
	* ionization efficiency problem.  background_rate is only here to provide the relationship
	* for predominantly neutral gas.  the background_density is the real rate. 
	hextra.background_density = 1.99e-9f;*/
	/* >>chng 05 apr 16, to get proper ionization rate in ism_set_cr_rate, where
	* H is forced to be fully atomic, no molecules, density from 1.99 to 2.15 */
	hextra.background_density = 2.15e-9f;
	hextra.background_rate = 2.5e-17f;

	/* initialization for punch files - must call after input commands have
	 * been scanned for grid and vary options.  So known if grid or single run 
	 * default punch is different for these */
	grid.lgGridDone = false;
	grid.lgStrictRepeat = false;

	PunchFilesInit();

	H2_init_coreload();

	return;
}
