/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */

#ifndef _MEAN_H_
#define _MEAN_H_

/**mean derive mean ionization fractions over computed EXTERN structure */
void MeanInc(void);

/**MeanZero zero mean of ionization fractions array */
void MeanZero(void);

/**MeanIonRadius derive mean ionization fractions over ravius for some element 
\param chType either 'i' or 't' for ionization or temperature
\param nelem atomic number on physical, no c, scale 
\param *n this will say how many of arlog have non-zero values
\param arlog[] array of values, log both cases
\param lgDensity true, include electron density, false do not
*/
void MeanIonRadius(
	char chType,
	long int nelem, 
	long int *n, 
	realnum arlog[],
	bool lgDensity );

/**MeanIonVolume do volume mean of ionization fractions over volume of any element 
\param chType either 'i' or 't' for ionization or temperature 
\param nelem atomic number on physical, no c, scale
\param *n this will say how many of arlog have non-zero values
\param arlog[] array of values, log both cases 
\param lgDensity true, include electron density, false do not
*/
void MeanIonVolume(
	char chType,
	long int nelem, 
	long int *n, 
	realnum arlog[],
	bool lgDensity );

/**aver compute average of various quantities over the computed geometry
 * called by startenditer to initialize, radinc to increment, and prtfinal for final results 
 \param *chWhat
 \param quan
 \param weight
 \param *chLabl
 */
void aver(
  const char *chWhat, 
  double quan, 
  double weight, 
  const char *chLabl);

/** ionmeans.h */
EXTERN struct t_mean {

	/** used to store information for mean ionization and electron temperature,
	 * this is set to zero in zmean, incremented in mean, and analysed in 
	 * MeanIonRadius and MeanIonVolume */

	/** this is used to get the harmonic mean temperature with respect to
	 * atomic hydrogen density, over radius, for comparison with 21cm data */
	double HarMeanTempRadius[2];
	/** this is used to get the harmonic mean temperature with respect to
	 * atomic hydrogen density, weighted over volume */
	double HarMeanTempVolume[2];

	/** used to get mean temp over radius and volumn */
	double TempMeanRadius[2] ,
		TempMeanVolume[2];

	/** mean B weighted by n(H0) / T as in 21 cm measures */
	double B_HarMeanTempRadius[2];

	/** this is used to get the mean temperature with respect to
	 * H2, over radius */
	double H2MeanTempRadius[2];
	/** this is used to get the mean temperature with respect to
	 * H2, weighted over volume */
	double H2MeanTempVolume[2];

	/** this is mean of computed 21 cm spin temperature */
	double H_21cm_spin_mean_radius[2];

	/** xIonMeans[type][nelem][stage]
	 * stage=0 is integrated sum of all
	 * type = 0 is radius integration, 1=vol integration */
	double ***xIonMeans;/**[2][LIMELM][LIMELM+2];*/
	/** following includes electron density */
	double ***xIonEdenMeans;/**[2][LIMELM][LIMELM+2];*/
	double **xIonMeansNorm;/**[2][LIMELM];*/
	/** following includes electron density */
	double **xIonEdenMeansNorm;/**[2][LIMELM];*/

	/** TempMeans[type][nelem][stage]
	 * stage=0 is not used, here for analogy with above
	 * type = 0 is radius integration, 1=vol integration,
	 * 2 is norm for radius, 3 is norm for vol integration */
	double ***TempMeans /*[4][LIMELM][LIMELM+2]*/;
	double ***TempMeansNorm;
	/** following includes electron density */
	double ***TempEdenMeans;
	double ***TempEdenMeansNorm;

	}	mean;


#endif /* _MEAN_H_ */
