/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */

#ifndef _OPTIMIZE_H_
#define _OPTIMIZE_H_


/** called instead of cloudy to do optimize routine,
 \return return value is 0 if calculation is ok, 1 if disaster */
bool lgOptimize_do(void);

/**vary_input sets input lines to feed into cloudy in optimization runs 
\param *lgLimOK
*/
void vary_input(bool *lgLimOK);

/** optimize_subplex is the main driver, and only exposed, routine for the
 * cowan downhill simplex routine 
 \param n
 \param tol
 \param maxnfe
 \param mode
 \param scale[]
 \param x[]
 \param *fx
 \param *nfe
 \param work[]
 \param iwork[]
 \param *iflag
 */
void optimize_subplex(long int n, 
	  double tol, 
	  long int maxnfe, 
	  long int mode, 
	  realnum scale[], 
	  realnum x[], 
	  realnum *fx, 
	  long int *nfe, 
	  realnum work[], 
	  long int iwork[], 
	  long int *iflag);

/*<NP->*/
/**\<I4\> default to INTEGER*4*/
#define	VRSNEW	3.00f
/** this is the name of the file that optimize_phymir automatically creates containing
 * information to continue the optimization at a later time.  */
/* >>chng 00 sep 12, had been continue.in but was confused with normal input files*/
#define CNTFILE "continue.pmr"

void optimize_phymir(realnum[],realnum[],long,realnum*,realnum);

/**optimize_func actual function called during evaluation of optimization run */
double optimize_func(realnum param[]);

/* varypar.h */
/** the limit to the number of numbers on the command line */
#define	LIMEXT	5L
#define	LIMPAR	20L
#define	NCOLLM	100L
#define	NOBSLM	100L

EXTERN char chOptimFileName[INPUT_LINE_LENGTH]; /**< file name for output, set in cddefines.c */

/**
 *
 *logical variable says whether current line image has vary option
 *
 *optimize increments, deltas for changing optimized variables
 *range for optimize command
 *io unit for final best parameters from optimizer
 *limit to number of iterations for optimizer, set with 
 *optimize iterations command
 *set with optimize tolerance command, used for global match to fit
 * default set in scalar to 0.10
 *current counter for the number of calls to the optimizer<BR>
 *lgTrOpt flag set with optimization trace command<BR>
 *nTrOpt is which call to cloudy to turn on trace<BR>
 *flags set if we are to optimize lines, luminosity, or colums<BR>
 *
 *
 *     labels for column densities on vary command<BR>
 *     this specifies the optimization routine<BR>
 *     'amoe', 'powe', 'bubr'
 */
EXTERN struct t_optimize {

	/** flag set true if vary option used */
	bool lgVaryOn,	
		/** flag set true if no vary command entered */
		lgNoVary;

	/** flag set true if optimize command entered */
	bool lgOptimr;

	realnum 
		/** min and max of range of variation */
	  varmax[LIMPAR], 
	  varmin[LIMPAR], 

	  /** vparm is the value of the parameters on the line ,
	   * there can be as many as LIMEXT parameters - only first is varied */
	  vparm[LIMEXT][LIMPAR], 

	  /** the increment */
	  vincr[LIMPAR];

	/** the number of numbers on the command line */
	long int nvarxt[LIMPAR], 
	  nvfpnt[LIMPAR];

	realnum vpused[LIMPAR], 
	  OptIncrm[LIMPAR], 
	  varang[LIMPAR][2];

	long int nvary, 
	  nparm,
	  nRangeSet;
	bool lgVarOn;
	realnum ColDen_Obs[NCOLLM], 
	  chColDen_error[NCOLLM];
	long int ion_ColDen[NCOLLM], 
	  ncobs,
	  ionTemp[NCOLLM];

	realnum xLineInt_Obs[NOBSLM], 
	  xLineInt_error[NOBSLM];

	realnum temp_obs[NOBSLM], 
	  temp_error[NOBSLM];
	char chTempWeight[NOBSLM][7];

	long int 
	  nlobs, 
	  nIterOptim,
	  /**< number of temps entered */
	  nTempObs;

	/** line wavelengths */
	realnum wavelength [NOBSLM],
		/** error on the wavelength */
		errorwave[NOBSLM];

	/** following will be file where best values will go */
	FILE *ioOptim;
	realnum OptGlobalErr;

	/* counter for number of models in a grid - is one (not zero) for 
	 * first simulation */
	long int nOptimiz;

	bool lgOptimFlow;
	realnum optint, 
	  optier;
	long int nTrOpt;
	bool lgTrOpt;
	bool lgOptimize, 
	  /** these flags say we are optimizing on line intensity, luminosity,
	   * column density, or temperature */
	  lgOptLin, 
	  lgOptLum, 
	  lgOptCol,
	  lgOptTemp;

	/** the following is needed by PHYMIR */
	bool lgParallel,
	  lgOptCont;
	long useCPU;

	char chVarFmt[LIMPAR][FILENAME_PATH_LENGTH_2], 
	  chColDen_label[NCOLLM][5], 
	  chLineLabel[NOBSLM][5], 
	  chTempLab[NOBSLM][5],
	  chOptRtn[5];

	double SavGenericData[10];

	}	optimize;



#endif /* _OPTIMIZE_H_ */
