/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */

#ifndef _PARSE_H_
#define _PARSE_H_


/** this is the assumed temperature of the cosmic background radiation<BR>
 >>refer	CMB	temp	Mather, J.C., Fixsen, D.J., Shafer, R.A., Mosier, C., & <BR>
 * >>refercon	Wilkinson, D.T. 1999, ApJ, 512, 511 */
#define	CMB_TEMP	2.725

/**ParseCommands main command line parser, called by Cloudy to decode commands, 
 * it then call other routines to parse specific commands */
void ParseCommands(void);

/**ParseCosmicRays parse the cosmic rays command 
\param *chCard
*/
void ParseCosmicRays( char *chCard );

/**ParseAbundances parse and read in composition as set by abundances command 
\param *chCard
\param lgDSet
*/
void ParseAbundances(char *chCard,
  bool lgDSet);

/**ParseDont parse the dont command */
void ParseDont(char*);

/**ParsePunch parse the punch command 
\param *chCard
*/
void ParsePunch(char *chCard );

/**initialize punch file pointers */
void PunchFilesInit(void);

/**close all open punch files 
\param lgFinal - close ALL files, regardless of "no clobber" status when true
*/
void ClosePunchFiles( bool lgFinal );

/**ParseAge - parse the age command */
void ParseAge(char*);

/**ParseAgn parse parameters for the AGN continuum shape command 
\param *chCard
*/
void ParseAgn(char *chCard);

/**ParseState save or recover previous state of the code 
\param *chCard
*/
void ParseState(char *chCard);

/** parse the blackgody command 
\param *chCard input command line, already changed to caps
\param *nqh counter for which continuum source this is
\param *ar1 optional area that might be set here
*/
void ParseBlackbody(
  char *chCard,	
  long int *nqh,
  realnum *ar1);					

/**ParseCompile compile werner or kurucz model atmospheres into cloudy format, by K Volk 
\param *chCard
*/
void ParseCompile(char *chCard );

/**ParseConstant parse the constant ... command */
void ParseConstant(char*);

/**ParseDLaw parse parameters on the dlaw command so set some density vs depth law
\param *chCard
*/
void ParseDLaw(char *chCard );

/**ParseTLaw parse parameters on the tlaw command to set some temperature vs depth 
\param *chCard
*/
void ParseTLaw(char *chCard );

/**ParseDrive parse the drive command - drive calls to various subs 
\param *chCard
*/
void ParseDrive(char *chCard );

/**ParseGrain parse parameters on Peter's version of the grains command 
\param *chCard
\param *lgDset
*/
void ParseGrain(char *chCard, 
  bool *lgDset);

/**ParseFluc parse the fluctuations command */
void ParseFluc(char*);

/**ParseHDEN parse the HDEN command */
void ParseHDEN(char*);

/**ParseAtomISO parse the atom XX-like command, to set options for iso sequences 
\param ipISO
\param *chCard
*/
void ParseAtomISO(long ipISO, char *chCard);

/**ParseAtomH2 parse information from the rotor command line 
\param *chCard
*/
void ParseAtomH2(char *chCard );

/**ParseAtomCO parse the atom co command 
\param *chCard
*/
void ParseAtomCO(char *chCard );

/**ParseGrid parse the grid command 
\param *chCard
*/
void ParseGrid(char *chCard);

/**ParseInit parse the init command */
void ParseInit(char*);

/**ParseInterp parse parameters on interpolate command 
\param *chCard
\param *lgEOF
*/
void ParseInterp(char *chCard, 
  bool *lgEOF);

/**ParseIonPar parse the ionization parameter command 
\param *nqh
\param *chCard
\param *ar1
*/
void ParseIonPar(long int *nqh, 
  char *chCard,
  realnum *ar1);

/**ParseNorm parse parameters on the normalize command 
\param *chCard
*/
void ParseNorm(char *chCard);

/**ParseOptimize parse the optimize command 
\param *chCard
*/
void ParseOptimize(char *chCard);

/**ParsePrint parse the print command  
\param *chCard
*/
void ParsePrint(char *chCard );

/**ParseRadius parse the radius command */
void ParseRadius(char*,realnum*);

/**ParseSet parse the set command */
void ParseSet(char*);

/**ParseTable parse the table read command 
\param *nqh
\param *chCard
\param *ar1
*/
void ParseTable(long int *nqh, 
  char *chCard,
  realnum *ar1);

/**ParseTrace parse the trace command */
void ParseTrace(char*);

/*ParseExtinguish parse the extinguish command */
void ParseExtinguish( char *chCard );

/*ParseCaseB - parse the Case B command */
void ParseCaseB( char *chCard );

/**ParseTest parse the test command */
void ParseTest(char* , 
			   long int *nqh,
			   realnum *ar1,
			   bool lgDSet);

/**ParseAbsMag parse the absolute magnitude command */
void ParseAbsMag(char*,long*);

/**ParseBackgrd parse the background continuum command */
void ParseBackgrd(long*,char*,realnum*);

/**ParseCoronal parse the cronal equilibrum command */
void ParseCoronal(char*,long*,realnum*);

/**ParseElement parse options on element command */
void ParseElement(char *chCard );

/**ParseCMB parse parameters from fireball command 
\param z
\param *nqh
\param *ar1
*/
void ParseCMB(double z, 
  long int *nqh, 
  realnum *ar1);

/**ParseF_nu parse intensity command parameters 
\param *chCard
\param *nqh
\param *ar1
\param *chType
\param lgNU2
*/
void ParseF_nu(
  char *chCard, 
  long int *nqh, 
  realnum *ar1, 
  const char *chType, 
  bool lgNU2);

/**ParseGlobule parse parameters off the globule command 
\param *chCard
*/
void ParseGlobule(char *chCard);

/**ParseRangeOption parse the range option on the luminosity command */
void ParseRangeOption(long,char*);

/**ParseMap parse map command to produce map of heating and cooling */
void ParseMap(char*);

/**ParseMetal parse parameters on metal command */
void ParseMetal(char*);

/**ParsePlot parse the plot command */
void ParsePlot(char*);

/**ParsePowerlawContinuum parse the power law continuum command */
void ParsePowerlawContinuum(char*);

/**ParseRatio parse the ratio command */
void ParseRatio(char*,long*);

/**ParseSphere parse the sphere command */
void ParseSphere(char*);

/**ParseStop parse the stop command */
void ParseStop(char*);

/**ParseCrashDo any of several tests to check that the code can crash 
\param *chCard
*/
void ParseCrashDo(char *chCard);

#endif /* _PARSE_H_ */
