/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*ParseAge parse parameters off the age command */
#include "cddefines.h"
#include "timesc.h"
#include "parse.h"

void ParseAge(char *chCard )
{
	bool lgEOL;
	long int i;

	DEBUG_ENTRY( "ParseAge()" );

	/* set age for the cloud
	 * various timescales will be checked in AgeCheck, called in comment */

	i = 4;
	timesc.CloudAgeSet = (realnum)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);

	/* key " off" turns age off */
	if( lgEOL && (!nMatch(" OFF",chCard)) )
	{
		fprintf( ioQQQ, " The age must be on this line.\n" );
		cdEXIT(EXIT_FAILURE);
	}

	/* check if log of age */
	if( nMatch(" LOG",chCard) )
	{
		timesc.CloudAgeSet = (realnum)pow((realnum)10.f,timesc.CloudAgeSet);
	}

	/* check for units, we want seconds in the end */
	if( nMatch("MILL",chCard) )
	{
		/* millennium */
		timesc.CloudAgeSet *= (realnum)(3.15569e7*1000.);
	}
	else if( nMatch("CENT",chCard) )
	{
		/* centuries */
		timesc.CloudAgeSet *= (realnum)(3.15569e7*100.);
	}
	else if( nMatch("YEAR",chCard) )
	{
		/* years */
		timesc.CloudAgeSet *= 3.15569e7;
	}
	else if( nMatch("MONT",chCard) )
	{
		/* months */
		timesc.CloudAgeSet = (realnum)(timesc.CloudAgeSet*3.15569e7/12.);
	}
	else if( nMatch("FORT",chCard) )
	{
		/* fortnights */
		timesc.CloudAgeSet *= 24.f*3600.f*14.f;
	}
	else if( nMatch("WEEK",chCard) )
	{
		/* weeks */
		timesc.CloudAgeSet *= 24.f*3600.f*7.f;
	}
	else if( nMatch("DAY ",chCard) )
	{
		/* days */
		timesc.CloudAgeSet *= 24.f*3600.f;
	}
	else if( nMatch("HOUR",chCard) )
	{
		/* hours */
		timesc.CloudAgeSet *= 3600.f;
	}
	else if( nMatch("MINU",chCard) )
	{
		/* minuts */
		timesc.CloudAgeSet *= 60.f;
	}
	else if( nMatch("SECO",chCard) )
	{
		/* seconds - this is the default */
		timesc.CloudAgeSet *= 1.f;
	}

	return;
}
