/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
* others.  For conditions of distribution and use see copyright notice in license.txt */
/*ParseCaseB - parse the Case B command */
#include "cddefines.h"
#include "opacity.h"
#include "parse.h"

/*ParseCaseB - parse the Case B command */
void ParseCaseB( char *chCard )
{
	long int i;
	bool lgEOL;

	DEBUG_ENTRY( "ParseTest()" );

	/* set flag saying we are doing case b */
	opac.lgCaseB = true;

	/* scan in an optional optical depth in lya */
	i = 5;
	opac.tlamin = (realnum)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
	if( lgEOL )
	{
		/* set tau-La to 10**9 if not specified 
		opac.tlamin = 1e9;*/
		/* >>chng 01 sep 24, change from 1e9 default to 1e5, to get
		* more realistic conditions in HII region.  Very large tau
		* caused extreme Lya behavior */
		opac.tlamin = 1e5;
	}
	else
	{
		opac.tlamin = (realnum)pow((realnum)10.f,opac.tlamin);
	}

	/* Hummer and Storey case B; no collisions from n=1, 2 (usually in) */
	if( nMatch("HUMM",chCard) )
	{
		opac.lgCaseB_HummerStorey = true;
	}

	/* the NO PHOTOIONIZATION option, turns off excited state photoionization */
	if( nMatch("O PH",chCard) )
	{
		opac.lgCaseB_no_photo = true;
	}

	/* the NO PDEST option, turns off line destruction by background opacities */
	if( nMatch("O PDE",chCard) )
	{
		/* turn off destruction off line destruction by background opacities */
		opac.lgCaseB_no_pdest = true;
	}
	return;
}
