/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*ParseCoronal parse parameters off coronal equilibrium command */
#include "cddefines.h"
#include "rfield.h"
#include "thermal.h"
#include "input.h"
#include "optimize.h"
#include "phycon.h"
#include "radius.h"
#include "dynamics.h"
#include "parse.h"

/*ParseCoronal parse parameters off coronal equilibrium command */
void ParseCoronal(char *chCard,
  long int *nqh, 
  realnum *ar1)
{
	bool lgEOL;
	long int i;
	double a;

	DEBUG_ENTRY( "ParseCoronal()" );

	if( nMatch( "INIT" , chCard ) && nMatch( "TIME" , chCard ) )
	{
		dynamics.lg_coronal_time_init = true;
	}

	/* coronal equilibrium; set constant temperature to number on line */
	thermal.lgTemperatureConstant = true;
	thermal.lgTemperatureConstantCommandParsed = true;
	i = 5;
	a = FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
	if( lgEOL )
	{
		fprintf( ioQQQ, " There should be a temperature on this line.\n" );
		cdEXIT(EXIT_FAILURE);
	}

	/* numbers less than or equal to 10 are the log of the temperature */
	if( a <= 10. && !nMatch("LINE",chCard) )
	{
		thermal.ConstTemp = (realnum)pow(10.,a);
	}
	else
	{
		thermal.ConstTemp = (realnum)a;
	}

	/* insure not below lowest allowed temperature for the code */
	if( thermal.ConstTemp < phycon.TEMP_LIMIT_LOW )
	{
		fprintf( ioQQQ, " temperature on coronal command cannot be below %.3f, reset to 3K.\n",
			phycon.TEMP_LIMIT_LOW);
		thermal.ConstTemp = 3.;
	}
	else if( thermal.ConstTemp > phycon.TEMP_LIMIT_HIGH )
	{
		fprintf( ioQQQ, " temperature on coronal command cannot be above %.3f, reset to 3e9K.\n",
			phycon.TEMP_LIMIT_HIGH);
		thermal.ConstTemp = 3e9;
	}

	/* now simulate a BREMS line */
	strcpy( rfield.chSpType[rfield.nspec], "BREMS" );
	rfield.slope[rfield.nspec] = 1e8;
	rfield.cutoff[rfield.nspec][0] = 0.;
	rfield.cutoff[rfield.nspec][1] = 0.;

	/* simulate an ionization parameter line */
	strcpy( rfield.chRSpec[*nqh], "SQCM" );
	strcpy( rfield.chSpNorm[*nqh], "IONI" );

	/* >>chng 96 jun 17, to stop mole network from crashing */
	/* >>chng 05 aug 15, this sets ionization parameter, in test case ism_hot_brems the
	 * value of 1e-10 was enough to dominate the ionization of he-like N - it's ionization
	 * then jumped due to large optical depth in the continuum - change U from -10 to -15 */
	/* >>chng 05 aug 16, this very strongly affected the coll_t4 sim - apparently there
	 * was a significant photoionization contribution from the -10 continuum,
	 * this was close to a 'no photoionization' case, but lower further to insure
	 * no photo contribution 
	 * chang from -15 to -20 */
	rfield.totpow[*nqh] = -20.f;

	/* set R to large value if U specified but R is not */
	/* set radius to very large value if not already set */
	/* >>chng 01 jul 24, from Radius == 0 to this, as per PvH comments */
	if( !radius.lgRadiusKnown )
	{
		*ar1 = (realnum)radius.rdfalt;
		radius.Radius = pow(10.,radius.rdfalt);
	}

	++rfield.nspec;
	if( rfield.nspec >= LIMSPC )
	{
		/* too many continua were entered */
		fprintf( ioQQQ, " Too many continua entered; increase LIMSPC\n" );
		cdEXIT(EXIT_FAILURE);
	}
	++*nqh;
	if( *nqh >= LIMSPC )
	{
		/* too many continua were entered */
		fprintf( ioQQQ, " Too many continua entered; increase LIMSPC\n" );
		cdEXIT(EXIT_FAILURE);
	}

	/* vary option */
	if( optimize.lgVarOn )
	{
		/*  no luminosity options on vary */
		optimize.nvarxt[optimize.nparm] = 1;
		strcpy( optimize.chVarFmt[optimize.nparm], "COROnal equilibrium %f" );

		/*  pointer to where to write */
		optimize.nvfpnt[optimize.nparm] = input.nRead;

		/*  log of temp will be pointer */
		optimize.vparm[0][optimize.nparm] = (realnum)log10(phycon.te);
		optimize.vincr[optimize.nparm] = 0.1f;
		++optimize.nparm;
	}
	return;
}
