/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*ParseExtinguish parse the extinguish command */
#include "cddefines.h"
#include "rfield.h"
#include "extinc.h"

/*ParseExtinguish parse the extinguish command */
void ParseExtinguish( char *chCard )
{
	long int i;
	bool lgEOL;
	/* extinguish ionizing continuum by absorbing column AFTER
	* setting luminosity or Q(H).  First number is the column
	* density (log), second number is leakage (def=0%)
	* last number is lowest energy (ryd), last two may be omitted
	* from right to left 
	* 
	* extinction is actually done in extin, which is called by ContSetIntensity */

	DEBUG_ENTRY( "ParseExtinguish()" );

	i = 5;
	extinc.excolm = (realnum)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
	if( lgEOL )
		NoNumb(chCard);

	/* >>chng 01 dec 19, add linear option */
	/* default is for the number to be the log of the column. 
	* there is a linear option for the column or optical depth,
	* if linear does not occur then log, and convert to linear */
	if( !nMatch("LINE" , chCard ) )
	{
		/* >>chng 03 jan 02, make comment if first value seems too large */
		if( extinc.excolm>35. )
		{
			fprintf(ioQQQ,
				" The first parameter on this command line is the log of either the column density or optical depth.\n");
			fprintf(ioQQQ,
				" The value seems pretty big to me - please check it.\n");
			/* flush it since we will probably crash */
			fflush(ioQQQ);
		}
		extinc.excolm = (realnum)pow((realnum)10.f,extinc.excolm);
	}

	/* option to set leakage - default is 0. */
	extinc.exleak = (realnum)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
	/* optional leakage is zero */
	if( lgEOL )
		extinc.exleak = 0.;

	/* negative leaks are logs */
	if( extinc.exleak < 0. )
		extinc.exleak = (realnum)pow((realnum)10.f,extinc.exleak);

	if( extinc.exleak > 1. )
	{
		/* but leaks greater than 1 are not allowed */
		fprintf( ioQQQ, " A leakage of%9.0f%% was entered - this must be less than 100%%\n", 
			extinc.exleak*100. );
		cdEXIT(EXIT_FAILURE);
	}
	/* user input check that H-ionizing radiation is blocked if
	* table Draine used */
	rfield.lgBlockHIon = true;

	/* option to set lowest energy for absorber */
	extinc.exlow = (realnum)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
	if( lgEOL )
	{
		extinc.exlow = 0.99946f;
	}

	else
	{
		if( extinc.exlow <= 0. )
			extinc.exlow = (realnum)pow((realnum)10.f,extinc.exlow);
		if( extinc.exlow < 0.99946 )
		{
			fprintf( ioQQQ, " Energy less than 1 Ryd!!\n" );
		}
	}

	/* >>chng 01 dec 19, add optical depth at 1 Ryd rather than column density */
	if( nMatch("OPTI" , chCard ) )
	{
		/* convert the optical depth into the proper column density */
		extinc.excolm /= (realnum)(extinc.cnst_col2optdepth*
			pow(extinc.exlow,extinc.cnst_power) );
	}
	return;
}
