/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*ParseGrid parse the grid command lines */
#include "cddefines.h"
#include "optimize.h"
#include "grid.h"

/* ParseGrid - called from ParseCommands if GRID command found */
void ParseGrid(
	/* command line, which was changed to all caps in main parsing routine */
	char *chCard)
{
	bool lgEOL;
	long int i;

	DEBUG_ENTRY( "ParseGrid()" );

	/* RP fake optimizer to run a grid of calculations, also accepts
	 * keyword XSPEC */
	strcpy( optimize.chOptRtn, "XSPE" );
	grid.lgGrid = true;

	if( nMatch("REPE",chCard) )
	{
		/* just keep repeating, don't actually change the values in the grid.
		 * useful for debugging unintentional crosstalk */
		grid.lgStrictRepeat = true;
	}

	/* 06 aug 22, change to accept three parameters: lower and upper limit and number of points. */
	/* scan off range for the previously selected variable */
	if( optimize.nparm > 0 )
	{
		realnum highvalue;
		long numPoints=0;

		ASSERT( optimize.nparm < LIMPAR );

		i = 5;
		optimize.varang[optimize.nparm-1][0] = (realnum)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
		optimize.varang[optimize.nparm-1][1] = (realnum)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
		grid.paramIncrements[optimize.nparm-1] = (realnum)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);

		if( grid.paramIncrements[optimize.nparm-1] <= 0. )
		{
			fprintf( ioQQQ," The increment (third parameter) must be a positive number.\n" );
			fprintf( ioQQQ," Sorry.\n" );
			cdEXIT( EXIT_FAILURE );
		}

		if( grid.paramIncrements[optimize.nparm-1] > ( optimize.varang[optimize.nparm-1][1] - optimize.varang[optimize.nparm-1][0] ) )
		{
			fprintf( ioQQQ," The increment (third parameter) must not be greater than the difference between the limits (first and second parameters.\n" );
			fprintf( ioQQQ," Sorry.\n" );
			cdEXIT( EXIT_FAILURE );
		}

		if( lgEOL )
		{
			fprintf( ioQQQ," This command has changed since the definition given in Porter et al. 2006, PASP, 118, 920.\n" );
			fprintf( ioQQQ," The grid command now requires three parameters: lower limit, upper limit, and increment.\n" );
			fprintf( ioQQQ," The keywords RANGE and STEPS are no longer necessary.\n" );
			fprintf( ioQQQ," Sorry.\n" );
			cdEXIT( EXIT_FAILURE );
		}
		else
		{
			++optimize.nRangeSet;
		}

		/* Swap if second range parameter is less than the first. */
		if( optimize.varang[optimize.nparm-1][1] < optimize.varang[optimize.nparm-1][0] )
		{
			realnum temp = optimize.varang[optimize.nparm-1][0];
			optimize.varang[optimize.nparm-1][0] = optimize.varang[optimize.nparm-1][1];
			optimize.varang[optimize.nparm-1][1] = temp;
		}

		ASSERT( optimize.varang[optimize.nparm-1][1] - optimize.varang[optimize.nparm-1][0] > 0. );

		{
			realnum integer, tempNumPoints = (optimize.varang[optimize.nparm-1][1] - 
				optimize.varang[optimize.nparm-1][0])/grid.paramIncrements[optimize.nparm-1];
			realnum fraction = modf( tempNumPoints, &integer );

			if( fraction > 0.99f || fraction < 0.01f )
			{
				optimize.varang[optimize.nparm-1][1] += 0.01f * grid.paramIncrements[optimize.nparm-1];
				fprintf( ioQQQ,"\n NOTE The range is very nearly (or exactly) an integer multiple of the increment.\n" );
				fprintf( ioQQQ," NOTE The upper limit has been increased by a small fraction of the increment to ensure that\n" );
				fprintf( ioQQQ," NOTE the specified upper limit is included in the grid.\n\n" );
			}
		}

		numPoints = 0;
		highvalue = optimize.varang[optimize.nparm-1][0];
		/* find the number of parameter values by incrementing from the lower limit up to the upper limit. */
		while( highvalue <= optimize.varang[optimize.nparm-1][1] )
		{
			++numPoints;
			highvalue += grid.paramIncrements[optimize.nparm-1];
		}

		grid.numParamValues[optimize.nparm-1] = numPoints;

		if( grid.numParamValues[optimize.nparm-1] < 2 )
		{
			fprintf( ioQQQ, " NOTE must have at least two grid points\n" );
		}
		else if( grid.numParamValues[optimize.nparm-1] > 20 )
		{
			fprintf( ioQQQ, " NOTE There are %li grid points.  Are you "
				"sure you want that many?\n" ,
				grid.numParamValues[optimize.nparm-1]);
		}

		grid.numParamValues[optimize.nparm-1] = MAX2( 2, grid.numParamValues[optimize.nparm-1] );
	}

	return;
}
