/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*ParseIonPar parse the ionization parameter command */
#include "cddefines.h"
#include "radius.h"
#include "optimize.h"
#include "rfield.h"
#include "input.h"
#include "parse.h"

void ParseIonPar(long int *nqh, 
  char *chCard,
  realnum *ar1)
{
	bool lgEOL;
	long int i;

	DEBUG_ENTRY( "ParseIonPar()" );

	/*  check not too many continua */
	if( *nqh >= LIMSPC )
	{
		/* too many continua were entered */
		fprintf( ioQQQ, " Too many continua entered; increase LIMSPC\n" );
		cdEXIT(EXIT_FAILURE);
	}

	/* this is counter for where to start scanning number on line - different
	 * for XI than for IONIZ */
	i = -1;
	/* say that continuum is per unit area, ionization parameter*/
	strcpy( rfield.chRSpec[*nqh], "SQCM" );
	if( chCard[0] == 'I' )
	{
		/* this is the usual ionization parameter, U */
		strcpy( rfield.chSpNorm[*nqh], "IONI" );
		i = 5;
	}
	else if( chCard[0] == 'X' )
	{
		/* the X-Ray ionization parameter, xi */
		strcpy( rfield.chSpNorm[*nqh], "IONX" );
		/* >>chng 06 jan 20, had used 5 as in ionization parameter so
		 * possible to miss beginning of number */
		i = 3;
	}
	else
	{
		fprintf(ioQQQ," ParseIonPar hit chCard insanity.\n");
		cdEXIT(EXIT_FAILURE);
	}

	/* get the ionization parameter*/
	rfield.totpow[*nqh] = FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
	if( lgEOL )
		NoNumb(chCard);

	/* check for linear option, if present take log since rfield.totpow[*nqh]
	 * being log ionization parameter is the default */
	if( nMatch( "LINE" , chCard ) )
		rfield.totpow[*nqh] = log10(rfield.totpow[*nqh]);

	/* >>chng 06 mar 22, add time option to vary only some continua with time */
	if( nMatch( "TIME" , chCard ) )
		rfield.lgTimeVary[*nqh] = true;

	/* vary option */
	if( optimize.lgVarOn )
	{
		if( chCard[0] == 'I' )
		{
			/* this is the usual ionization parameter, U */
			strcpy( optimize.chVarFmt[optimize.nparm], "IONIZATION PARAMETER=%f" );
		}
		else if( chCard[0] == 'X' )
		{
			/* the X-Ray ionization parameter, xi */
			strcpy( optimize.chVarFmt[optimize.nparm], "XI=%f" );
		}
		else
		{
			fprintf( ioQQQ, " Insanity in detecting which ionization parameter.\n" );
			cdEXIT(EXIT_FAILURE);
		}
		/* pointer to where to write */
		optimize.nvfpnt[optimize.nparm] = input.nRead;
		optimize.vparm[0][optimize.nparm] = (realnum)rfield.totpow[*nqh];
		optimize.vincr[optimize.nparm] = 0.5;
		optimize.nvarxt[optimize.nparm] = 1;
		++optimize.nparm;
	}

	/* set R to large value if U specified. */
	/* set radius to very large value if not already set */
	/* >>chng 01 jul 24, from Radius == 0 to this, as per PvH comments */
	if( !radius.lgRadiusKnown )
	{
		*ar1 = (realnum)radius.rdfalt;
		radius.Radius = pow(10.,radius.rdfalt);
	}
	/* increment nmber of specifications of continuum intensities, */
	++*nqh;
	return;
}
