/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*ParseTrace read in options off the trace command line */
#include "cddefines.h"
#include "iterations.h"
#include "geometry.h"
#include "trace.h"
#include "parse.h"

void ParseTrace(char *chCard )
{
	bool lgEOL;
	long int i;

	DEBUG_ENTRY( "ParseTrace()" );

	/* turn on trace at a certain zone; .LE.0 or none for starting trace
	 * optional second number is iteration to start debugger */

	/* generate string that says debug turned on - this is caught by perl script 
	 * if "no print" occurs on line do not print it - there is one sim in the
	 * test suite which tests debug print and do not want to trigger comment
	 * that debug prints are accidentally turned on */
	if( !nMatch("O PR",chCard) )
		fprintf(ioQQQ,"DEBUG trace output turned on.\n");

	/* set initially false, in case we do not turn on trace until
	 * later iteration or zone */
	trace.lgTrace = false;
	i = 5;
	/* this is which zone to turn on */
	trace.nznbug = (long)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
	if( lgEOL )
		trace.lgTrace = true;

	/* this is which iteration to turn on */
	trace.npsbug = (long)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
	if( lgEOL )
		trace.npsbug = 1;

	/* turn trace on now if no numbers on line */
	if( trace.nznbug == 0 && trace.npsbug <= 1 )
	{
		trace.lgTrace = true;
		geometry.nprint = 1;
		iterations.IterPrnt[0] = 1;
	}

	/* trace convergence is a special command, 
	 * only convergence loops, not full trace */
	if( nMatch("CONV",chCard) )
	{
		/* check for keyword, if not present
		 * then set to very high level of debugging - initially set to negative number, a sign
		 * that trace is not on yet, but to turn on trace convergence when we hit the right zone */
		/* 1 ConvPresTempEdenIoniz */
		if( nMatch("PRES",chCard) )
			trace.nTrConvg = -1;
		/* 2 ConvTempEdenIoniz*/
		else if( nMatch("TEMP",chCard) )
			trace.nTrConvg = -2;
		/* 3 ConvEdenIoniz*/
		else if( nMatch("EDEN",chCard) )
			trace.nTrConvg = -3;
		/* 4 ConvIoniz*/
		else if( nMatch("IONI",chCard) )
			trace.nTrConvg = -4;
		/* 5 below ConvBase*/
		/* > 5 all levels*/
		else
			trace.nTrConvg = -100;

		/* above set trace level to negative number - this will not trigger
		 * trace output - turn trace on now if no zone or iteration on line */
		if( trace.nznbug == 0 && trace.npsbug <= 1 )
			trace.nTrConvg *= -1;

		/* turn off normal trace parameters, this is a special case */
		trace.lgTrace = false;
		/*trace.nznbug = 10000;*/
		geometry.nprint = 10000;
		iterations.IterPrnt[0] = 10000;

		/* this is an option to also turn on ots rate debug prints */
		if( nMatch(" OTS",chCard) )
			trace.lgOTSBug = true;

		/* this is an option to also turn on electron density source debug prints,
		 * key is ESOURCE */
		if( nMatch("ESOU",chCard) )
			trace.lgESOURCE = true;
	}

	/* trace he-like and h-like must come early since they may have name of element */
	/* the trace h-like hydrogenic species command, with lots of options */
	if( nMatch("H-LI",chCard) )
	{
		/* turn on short trace for h-like species */
		trace.lgHBug = true;

		/* option to turn on full printout */
		if( nMatch("FULL",chCard) )
		{
			trace.lgIsoTraceFull[ipH_LIKE] = true;
		}
		else
		{
			trace.lgIsoTraceFull[ipH_LIKE] = false;
		}

		/* look for one of the element names on the line*/
		trace.ipIsoTrace[ipH_LIKE] = GetElem(chCard);

		/* if no element appears on the line GetElem fcn returns -1,
		 * in this case we want to do hydrogen */
		trace.ipIsoTrace[ipH_LIKE] = MAX2(0, trace.ipIsoTrace[ipH_LIKE] );
	}

	/* the trace h-like hydrogenic species command, with lots of options */
	if( nMatch("HE-L",chCard) )
	{
		/* turn on short trace for helium - like species */
		trace.lgHeBug = true;

		/* option to turn on full printout */
		if( nMatch("FULL",chCard) )
			trace.lgIsoTraceFull[ipHE_LIKE] = true;
		else
			trace.lgIsoTraceFull[ipHE_LIKE] = false;

		/* look for one of the element names on the line*/
		trace.ipIsoTrace[ipHE_LIKE] = GetElem(chCard);

		/* if no element appears on the line fcn returns -1,
		 * in this case we want to do helium */
		trace.ipIsoTrace[ipHE_LIKE] = MAX2(1, trace.ipIsoTrace[ipHE_LIKE] );
	}

	/* were there any keywords on the line? */
	if( nMatch("BETA",chCard) )
		trace.lgTr8446 = true;

	if( nMatch("CARB",chCard) )
		trace.lgCarBug = true;

	if( nMatch("CALC",chCard) )
		trace.lgCalBug = true;

	if( nMatch("COMP",chCard) )
		trace.lgComBug = true;

	if( nMatch("CONT",chCard) )
		trace.lgConBug = true;

	if( nMatch("COOL",chCard) )
		trace.lgCoolTr = true;

	if( nMatch("DIFF",chCard) )
		trace.lgTrDiff = true;

	if( nMatch(" DR ",chCard) )
		trace.lgDrBug = true;

	if( nMatch("EDEN",chCard) || nMatch("ELECTRON",chCard) )
		trace.lgNeBug = true;

	if( nMatch("GAUN",chCard) )
		trace.lgTrGant = true;

	if( nMatch("GRAI",chCard) )
		trace.lgDustBug = true;

	if( nMatch("HEAV",chCard) )
		trace.lgHeavyBug = true;

	if( nMatch("HEAT",chCard) )
		trace.lgHeatBug = true;

	/* trace helium, but not h-like or he-like */
	if( nMatch("HELI",chCard) && !nMatch("H-LI",chCard)  && !nMatch("HE-L",chCard) )
		trace.lgHeBug = true;

	/* the simple trace hydrogen command */
	if( nMatch("HYDR",chCard) && !nMatch("H-LI",chCard))
	{
		trace.lgHBug = true;
		trace.lgIsoTraceFull[ipH_LIKE] = false;
		/* this says which element, on the C scale (H=0), to trace */
		trace.ipIsoTrace[ipH_LIKE] = 0;
	}

	if( nMatch("IRON",chCard) )
		trace.lgFeBug = true;

	if( nMatch("VELN",chCard) )
		trace.lgTrLevN = true;

	if( nMatch("LINE",chCard) )
		trace.lgTrLine = true;

	if( nMatch("NEON",chCard) )
		trace.lgNeonBug = true;

	if( nMatch("MOLE",chCard) )
	{
		/* turn on molecular trace - either CO or H2, or both,
		 * keyword can be CMOLE or HMOLE depending on what is needed */
		if( nMatch("HMOL",chCard) )
		{
			/* H2 network */
			trace.lgTr_H2_Mole = true;
		}
		else if( nMatch("CMOL",chCard ) )
		{
			/* CO network */
			trace.lgTr_CO_Mole = true;
		}
		else
		{
			/* turn them both on */
			trace.lgTr_CO_Mole = true;
			trace.lgTr_H2_Mole = true;
		}
	}

	/* trace pointers */
	if( nMatch("POIN",chCard) )
		trace.lgPointBug = true;

	/* following two are optical, optimize */
	if( nMatch("PTIC",chCard) )
		trace.lgOptcBug = true;

	if( nMatch("PTIM",chCard) )
		trace.lgTrOptm = true;

	if( nMatch(" OTS",chCard) )
		trace.lgOTSBug = true;

	if( nMatch("SECO",chCard) && nMatch("IONI",chCard) )
		/* secondary ionization */
		trace.lgSecIon = true;

	if( nMatch("THRE",chCard) )
		trace.lgTrace3Bod = true;

	/* two photon emission, spontaneous and induced */
	if( nMatch(" TWO",chCard) )
		trace.lgBug2nu = true;

	/* wind geometry */
	if( nMatch("WIND",chCard) )
		trace.lgWind = true;

	/* falling through is fine - just turn on minimal trace */
	return;
}
