/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */

#ifndef _PUNCH_H_
#define _PUNCH_H_

/* punch.h */
#define	LIMPUN	100L

/**PunchDo produce punch output during calculation 
\param chTime chTime is null terminated 4 char string, either "MIDL" or "LAST" 
*/
void PunchDo(
	const char *chTime); 

/** punch one line, called by PunchLineStuff 
\param t
\param io
\param xLimit
\param index
\param population
*/
void pun1Line( 
	transition * t , 
	FILE * io , 
	realnum xLimit , 
	long index , 
	double population );

/**PunchLineData punches selected line data for all lines transferred in code 
\param io
*/
NORETURN void PunchLineData(
	FILE * io);

/**punch_opacity punch total opacity in any element, punch opacity command 
\param io
\param np
*/
void punch_opacity(
	FILE * io, 
	long int np);

/**PunchSpecial generate output for the punch special command 
\param io
\param chTime
*/
void PunchSpecial(
	FILE* io , 
  const char *chTime);

/**Punch1LineData data for punch one line 
\param t
\param io
\param lgCS_2 this flag says whether collision strength should be punched - should be false
	for multi level atoms since sums are not done properly 
*/
void Punch1LineData( 
	transition * t , 
	FILE * io,
	 bool lgCS_2 );

/**punch_line parse punch lines command, or actually do the punch output 
\param ip the file we will write to 
\param chDo
\param lgLog3
*/
void punch_line(
	FILE * ip,
  const char *chDo, 
  /* true, return rel intensity, false, log of true intensity */
  bool lgLog3,
  char *chHeader);

/**punch_average parse punch average command, or actually do the punch output 
\param ipPun - array index for file for final punch output 
\param chJob - the job we shall do, READ for reading from input stream, PUNCH 
*/
void punch_average( 
	long int ipPun,
	const char *chJob,
	char *chHeader );

/** punch_colden parse punch column density command, or actually do the 
punch lines output 
\param ip the file we will write to 
\param chDo
*/
void punch_colden(
  /* the header for the file - the identification for the column densities */
  char header[] ,
  /* the file we will write to */
  FILE * ioPUN, 
  /* the job we shall do - one of */
  const char *chDo );

/**Punch_Line_RT parse the punch line rt command - read in a set of lines 
\param ip the file we will write to
\param *chJob
*/
void Punch_Line_RT(
	FILE * ip, 
	const char *chJob );

/** Punch spectra to a FITS compatible file. 
\param io
\param option
*/
void punchFITSfile( 
	FILE* io, 
	int option );

/**HeatPunch punch contributors to local heating, with punch heat command, called by punch_do 
\param io
*/
void HeatPunch(FILE* io);

/**CoolPunch punch coolants, called by punch_do 
\param io
*/
void CoolPunch(FILE * io);

EXTERN struct t_punch {

	/** following are for punch LineList option
	 * nLineList is number of em lines, -1 if not defined */
	long int *nLineList;
	/** chLineListLabel is label for line list */
	char ***chLineListLabel;
	/** wlLineList is set of emission lines for LineList */
	realnum **wlLineList;
	/** flag saying whether to take ratio (true) of pairs */
	bool *lgLineListRatio;

	/** following are for punch averages option
	* nAverageList is number of averages, -1 if not defined */
	long int *nAverageList;
	/** chAverageType is label for type of average */
	char ***chAverageType;
	/** chAverageSpeciesLabel is label for species */
	char ***chAverageSpeciesLabel;
	/** nAverageIonList is set of ions for averages */
	int **nAverageIonList;
	/** nAverage2ndPar is set of second parameters for averages */
	int **nAverage2ndPar;

	/** this is the file where we will direct the output */
	FILE *ipPnunit[LIMPUN]; 

	/** is this a real punch command, or one of the similar options like
	 * punch dr, which is not done in punch files */
	bool lgRealPunch[LIMPUN]; 

	/** number of punch commands entered */
	long int npunch;

	/**chPunch - what is it we want to punch? set in GetPunch, used in DoPunch */
	char chPunch[LIMPUN][5];

	/** which opacity to punch out  */
	char chOpcTyp[LIMPUN][5];

	/** this flag tells us whether to punch results of a grid to separate files
	 * for each grid point or all to the same file.  Different for different 
	 * punch commands */
	bool lgPunchToSeparateFiles[LIMPUN];

	/** option to not insert end-of-iteration separator - used for punch files
	 * that create one line per iteration */
	bool lg_separate_iterations[LIMPUN];

	/** flag saying whether we should punch headers.  Used in grid punches so that
	 * the header only gets punched once. */
	bool lgPunHeader;

	/** flag saying whether any punch continuum commands were entered
	 * set true in parsecontinuum when punch continuum entered,
	 * used in PrtComment to warn if continuum punched with no iterations */
	bool lgPunContinuum;

	/** punarg is set of optional arguments for the punch command */
	realnum punarg[LIMPUN][3];

	/** implement punch every option - lgPunchEveryZone true if want to punch
	 *every zone, nPunchEveryZone is number of zones to punch */
	bool lgPunchEveryZone[LIMPUN];
	long int nPunchEveryZone[LIMPUN];

	/** set of optional arguments for punch command, but as a string */
	char chPunchArgs[LIMPUN][5];

	/** lg flag lgPunLstIter for this punch option,
	 do we only want to punch on last iteration? */
	bool lgPunLstIter[LIMPUN];

	/** flag saying that this punch file is in FITS format */
	bool lgFITS[LIMPUN];

	/** option to say whether any FITS output should be punched, 
	 * initialized to false, but turned to true after last grid exec.  */
	bool lgPunchFits;

	/**chConPunEnr - units of continuum in punch output */
	char chConPunEnr[LIMPUN][5];

	/** this global variable is index of punch command loop in dopunch */
	long int ipConPun;

	/** should hash marks be printed after every iteration?
	 * default is yes, set no with no hash option on punch command */
	bool lgHashEndIter[LIMPUN];

	/** this is the hash string, normally a set of hash marks, can be reset with
	 * set punch hash command */
	char chHashString[INPUT_LINE_LENGTH];

	/** flush file after every iteration */
	bool lgFLUSH;

	/** this is a prefix that will be set at the start of all punch file names
	 * set punch prefix command, normally a null string */
	char chFilenamePrefix[INPUT_LINE_LENGTH];

	/** these are initialized in parsepunch.c, in routine PunchFilesInit
	    min and max range of punch new continuum command */
	realnum cp_range_min[LIMPUN] ,
		/**max range of punch new continuum command*/ cp_range_max[LIMPUN];

	/** the resolving power of the rebinned continuum, set with
	 * set cpunch resolving 100 command, 
	 * if not specified then native resolution will be kept */
	realnum cp_resolving_power[LIMPUN];

	/** number of new punch continuum commands */
	long int cp_npun;

	/** set with punch line intensities and punch results commands,
	 * says whether results arrays produced by routine PunResults1Line should
	 * be column or array */
	char chPunRltType[7];

	/** option to punch out pointers with punch pointers command
	 * ipPoint is punch file handle, lgPunPoint says whether we will do it
	 */
	FILE* ipPoint;
	bool lgPunPoint;

	/** unit number, and flag, for punching reason for continued iterations */
	bool lgPunConv;
	FILE* ipPunConv;

	/** these control punching choice of dr - this is not really a punch command
	 * ipDRout is io unit, lgDROn says punching dr logic has been set,
	 * and lgDRPLst says to punch the last iteration */
	FILE * ipDRout;
	bool lgDROn, 
	  lgDRPLst,
	  lgDRHash;

	/* set true punch convergence base */
	bool lgTraceConvergeBase,
		lgTraceConvergeBaseHash;
	FILE * ipTraceConvergeBase;

	/** option to punch recombination coefficients to external file */
	FILE* ioRecom;
	bool lgioRecom;

	/** option to punch line intensities every for every zone
	 * logical variable says whether LinEvery was set */
	long int LinEvery;
	bool lgLinEvery;

	/** set skip sets this variable, which says how many cells to skip in punch */
	long int ncPunchSkip;

	/** threshold for faintest cooling or heating to be punched
	 * default is set to 0.05 in scalar, can be reset with 'set weakheatcool' */
	realnum WeakHeatCool;

	/** contrast factor for lines to continuum in punch output
	* default is 1 (gives correct line intensities) and changed with
	* set width command (enters width in km/sec) */
	realnum PunchLWidth;

	}	punch;

#endif /* _PUNCH_H_ */
