/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*punch_colden parse punch column density command, or actually do the punch lines output */
#include "cddefines.h"
#include "cddrive.h"
#include "input.h"
#include "punch.h"
/* this is the limit to the number of lines we can store */
#define	NPUNLM	100L

/*punch_colden parse punch column density command, or actually do the punch lines output */
void punch_colden(
  /* the header for the file, a list of identifications */
  char chHeader[] ,
  /* the file we will write to */
  FILE * ioPUN, 
  /* the job we shall do - one of READ or PUNS */
  const char *chDo )
{
	char chCAP[INPUT_LINE_LENGTH], 
	  chCard[INPUT_LINE_LENGTH];
	char chTemp[INPUT_LINE_LENGTH];

	bool lgEOF, 
	  lgEOL;
	long int i;
	static char chElement[NPUNLM][5];
	static long int nColdenEntered;
	static long int ionstage[NPUNLM];

	DEBUG_ENTRY( "punch_colden()" );

	if( strcmp(chDo,"READ") == 0 )
	{
		/* very first time this routine is called, chDo is "READ" and we read
		 * in lines from the input stream.  The line labels and wavelengths
		 * are store locally, and output in later calls to this routine */

		/* number of lines we will save */
		nColdenEntered = 0;

		/* get the next line, and check for eof */
		input_readarray(chCard,&lgEOF);
		if( lgEOF )
		{
			fprintf( ioQQQ, 
				" Hit EOF while reading line list; use END to end list.\n" );
			cdEXIT(EXIT_FAILURE);
		}

		/* convert line to caps */
		strcpy( chCAP, chCard );
		caps(chCAP);

		while( strncmp(chCAP, "END" ,3 ) != 0 )
		{
			if( nColdenEntered >= NPUNLM )
			{
				fprintf( ioQQQ, 
					" Too many lines have been entered; the %ld limit is.  Increase variable NPUNLM in routine punch_colden.\n", 
				  NPUNLM );
				cdEXIT(EXIT_FAILURE);
			}

			/* order on line is label (col 1-4), ionstage */
			strncpy( chElement[nColdenEntered], chCard , 4 );

			/* null terminate the string*/
			chElement[nColdenEntered][4] = 0;

			/* now get ionstage - 1 for atom, 2 for first ion, etc */
			i = 5;
			ionstage[nColdenEntered] = (long)FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
			if( lgEOL )
				NoNumb( chCard );

			/* this is total number stored so far */
			++nColdenEntered;

			/* get next line and check for eof */
			input_readarray(chCard,&lgEOF);
			if( lgEOF )
			{
				fprintf( ioQQQ, " Hit EOF while reading line list; use END to end list.\n" );
				cdEXIT(EXIT_FAILURE);
			}

			/* convert it to caps */
			strcpy( chCAP, chCard );
			caps(chCAP);
		}

		/*fprintf( ioPUN, "%li lines were entered, they were;\n", 
		  nColdenEntered );*/
		/* give header line */

		sprintf( chHeader , "#colden %s %3li", chElement[0] , ionstage[0] );
		for( i=1; i < nColdenEntered; i++ )
		{
			sprintf( chTemp, "\t%s %3li", chElement[i] , ionstage[i] );
			strcat( chHeader, chTemp );
		}
		strcat( chHeader, "\n" );
	}

	else if( strcmp(chDo,"PUNS") == 0 )
	{
		/* punch some column densities  */
		double colden;
		/* punch some column column densities command */
		for( i=0; i < nColdenEntered; i++ )
		{
			if( i )
				fprintf(ioPUN,"\t");
			/* get column density, returns 0 if all ok */
			if( cdColm(
				/* four char string, null terminated, giving the element name */
				chElement[i], 
				/* IonStage is ionization stage */
				ionstage[i], 
				/* will be column density */
				&colden) )
			{
				fprintf( ioQQQ, 
					"\n PROBLEM punch_colden could not find a column density for "
					"the species with label %s %li \n\n",
					chElement[i] , ionstage[i] );
				colden = 1.;
			}
			fprintf( ioPUN, "%.4f", log10( MAX2(SMALLFLOAT , colden ) ) );
		}
		fprintf( ioPUN, "\n" );
	}

	else
	{
		fprintf( ioQQQ, 
			" unrecognized key for punch_colden=%4.4s\n", 
		  chDo );
		cdEXIT(EXIT_FAILURE);
	}

	return;
}
