/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*RT_line_all do escape and destruction probabilities for all lines in code.  
 * Called with false arg by ionize, to only redo destruction probabilities,
 * and with true by cloudy to do both escape and destruction */
#include "cddefines.h"
#include "taulines.h"
#include "atomfeii.h"
#include "dense.h"
#include "conv.h"
#include "atoms.h"
#include "rfield.h"
#include "wind.h"
#include "iso.h"
#include "h2.h"
#include "opacity.h"
#include "trace.h"
#include "lines_service.h"
#include "atmdat.h"
#include "hydrogenic.h"
#include "rt.h"

void RT_line_all(
	/* this is true if we want to do both escape and destruction probabilities,
	 * and false if only destruction probabilities are needed */
	bool lgDoEsc ,
	/* flag saying whether to update fine opacities */
	bool lgUpdateFineOpac )
{
	long int i,
		ion,
		ipISO,
		nelem;
	long ipHi , ipLo;
	double factor,
		coloi;
	double SaveLyaPesc[NISO][LIMELM] , 
		SaveLyaPdest[NISO][LIMELM];
	/* this flag says whether to update the line RT */
	bool lgRT_update = lgUpdateFineOpac || lgDoEsc || conv.lgIonStageTrimed;

	DEBUG_ENTRY( "RT_line_all()" );

	if( trace.lgTrace )
		fprintf( ioQQQ, "     RT_line_all called\n" );

	/* skip line transfer if requested, and not first call in this zone */
	if( !rfield.lgDoLineTrans && conv.nPres2Ioniz )
	{
		return;
	}

	/* this array is huge and takes significant time to zero out or update, 
	 * only do so when needed, */
	rfield.lgFine_opac_update = lgUpdateFineOpac;
	if( rfield.lgFine_opac_update )
	{
		/* zero out fine opacity array */
		memset(rfield.fine_opac_zone , 0 , (unsigned long)rfield.nfine*sizeof(realnum) );

		/* this is offset within fine opacity array caused by Doppler shift
		 * between first zone, the standard of rest, and current position 
		 * in case of acceleration away from star in outflowing wind 
		 * dWind is positive, this means that the rest frame original
		 * velocity is red shifted to lower energy */
		realnum dWind = wind.windv - wind.windv0;
		/* will add ipVelShift to index of original energy so redshift has 
		 * to be negative */
		rfield.ipFineConVelShift = -(long int)(dWind / rfield.fine_opac_velocity_width+0.5);
	}

#	if 0
	/* this code is a copy of the code within line_one that does cloaking
	 * within this zone.  it can be used to see how a particular line
	 * is being treated. */
	{
#include "doppvel.h"
		double dTau , aa;
		ipISO = 0; nelem = 0;ipLo = 0;
		ipHi = 23;
		dTau =  Transitions[ipISO][nelem][ipHi][ipLo].Emis->PopOpc * 
			Transitions[ipISO][nelem][ipHi][ipLo].Emis->opacity / 
			DoppVel.doppler[nelem] + opac.opacity_abs[Transitions[ipISO][nelem][ipHi][ipLo].ipCont-1];
		dTau *= radius.drad_x_fillfac_mean;
		aa = log(1. + dTau ) / SDIV(dTau);
		fprintf(ioQQQ,"DEBUG dTau\t%.2f\t%.5e\t%.5e\t%.5e\n",fnzone,dTau,
			radius.drad_x_fillfac_mean,
			 aa);
	}
#	endif

	/* find Stark escape probabilities for hydrogen itself */
	RT_stark();

	/*if( lgUpdateFineOpac )
		fprintf(ioQQQ,"debuggg\tlgUpdateFineOpac in rt_line_all\n");*/
	for( ipISO=ipH_LIKE; ipISO<NISO; ++ipISO )
	{
		/* loop over all iso-electronic sequences */
		for( nelem=ipISO; nelem<LIMELM; ++nelem )
		{
			/* parent ion stage, for H is 1, for He is 1 for He-like and 
			 * 2 for H-like */
			ion = nelem+1-ipISO;

			/* element turned off */
			if( !dense.lgElmtOn[nelem] )
				continue;
			/* need we consider this ion? */
			if( ion <=dense.IonHigh[nelem] )
			{
				/* save escape and destruction probs for each Lya since
				 * these are special cases, and quantities are not damped
				 * in routine that evaluates each, test is to avoid
				 * evaluating iso sequence that does not exist for an element,
				 * as in He-like hydrogen */
				if( ipISO<=nelem )
				{
					SaveLyaPesc[ipISO][nelem] = Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pesc;
					SaveLyaPdest[ipISO][nelem] = Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pdest;
				}

				/* convert pops to per unit vol rather than per ion */
				if( dense.xIonDense[nelem][ion] > 1e-30 )
				{
					factor = dense.xIonDense[nelem][ion];
				}
				else
				{
					/* case where almost no parent ion - this will make
					 * very large line opacity, so background dest small */
					factor = 1.;
				}

				/* loop over all lines */
				for( ipHi=1; ipHi < iso.numLevels_local[ipISO][nelem]; ++ipHi )
				{
					for( ipLo=0; ipLo < ipHi; ++ipLo )
					{
						/* negative ipCont means this is not a real line, so do not
						 * transfer it */
						if( Transitions[ipISO][nelem][ipHi][ipLo].ipCont < 0 ) 
							continue;

						double SavePopOpc = Transitions[ipISO][nelem][ipHi][ipLo].Emis->PopOpc;
						/* must temporarily make ipLnPopOpc physical */
						Transitions[ipISO][nelem][ipHi][ipLo].Emis->PopOpc *= factor;
						/*if( ipISO==0 && nelem==0 && ipHi==12 && ipLo==11 )
							fprintf(ioQQQ,"DEBUG rt loop %li %li %li %li %.3e %.3e %.3e\n",
							ipISO, nelem, ipHi , ipLo , 
							Transitions[ipISO][nelem][ipHi][ipLo].Emis->PopOpc , 
							Transitions[ipISO][nelem][ipHi][ipLo].Emis->TauIn,
							Transitions[ipISO][nelem][ipHi][ipLo].Emis->TauTot);*/

						/* generate escape prob, pumping rate, destruction prob, 
						 * inward outward fracs */
						RT_line_one(&Transitions[ipISO][nelem][ipHi][ipLo] , lgDoEsc , lgUpdateFineOpac,true);


						{
							/* set true to print pump rates*/
							enum {DEBUG_LOC=false};
							if( DEBUG_LOC && nelem==1&& ipLo==0  /*&& iteration==2*/ )
							{
								fprintf(ioQQQ,"DEBUG pdest\t%3li\t%.2f\t%.3e\n",
									ipHi ,
									fnzone,
									Transitions[ipISO][nelem][ipHi][ipLo].Emis->Pdest);
							}
						}

						/* go back to original units so that final correction ok */
						Transitions[ipISO][nelem][ipHi][ipLo].Emis->PopOpc = SavePopOpc;
					}
				}

				if( ipISO > 0 && lgDoEsc )
				{
					/* do not let too much Lya escape outward since so important 
					 * H Lya is special case done above */
					Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pesc *= 
						opac.ExpmTau[Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].ipCont-1];
				}

				/* now update two photon induced rates */
				atmdat_2phot_rate(ipISO , nelem);

				/* this is option to not do destruction probabilities
				 * for case b no pdest option */
				if( opac.lgCaseB_no_pdest )
				{
					ipLo = 0;
					/* okay to let this go to numLevels_max. */
					for( ipHi=ipLo+1; ipHi < iso.numLevels_max[ipISO][nelem]; ++ipHi )
					{
						if( Transitions[ipISO][nelem][ipHi][ipLo].ipCont <= 0 )
							continue;

						/* hose the previously computed destruction probability */
						Transitions[ipISO][nelem][ipHi][ipLo].Emis->Pdest = SMALLFLOAT; 
					}
				}

				ipLo = 0;
				/* these are the extra lyman lines for the iso sequences */
				/*for( ipHi=2; ipHi < iso.nLyman[ipISO]; ipHi++ )*/
				/* only update if significant abundance and need to update fine opac */
				if( dense.xIonDense[nelem][ion] > 1e-30 && lgUpdateFineOpac )
				{
					/* we just want the population of the ground state */
					factor = StatesElem[ipISO][nelem][0].Pop * dense.xIonDense[nelem][ion];
					for( ipHi=StatesElem[ipISO][nelem][iso.numLevels_max[ipISO][nelem]-1].n; ipHi < iso.nLyman[ipISO]; ipHi++ )
					{
						/* must make ipLnPopOpc physical */
						ExtraLymanLines[ipISO][nelem][ipHi].Emis->PopOpc = factor;
						ExtraLymanLines[ipISO][nelem][ipHi].Lo->Pop =
							StatesElem[ipISO][nelem][ipLo].Pop;

						/* actually do the work */
						RT_line_one(&ExtraLymanLines[ipISO][nelem][ipHi] , lgDoEsc , lgUpdateFineOpac,true);

						/* reset to funny population of ground state */
						ExtraLymanLines[ipISO][nelem][ipHi].Emis->PopOpc = 
							StatesElem[ipISO][nelem][0].Pop;
					}
				}

			}/* if nelem if ion <=dense.IonHigh */
		}/* loop over nelem */
	}/* loop over ipISO */

	
	/* add on Stark escape probabilities for H itself */
	for( ipISO=0; ipISO<NISO; ipISO++ )
	{
		for( nelem=ipISO; nelem<LIMELM; nelem++ )
		{
			if( nelem < 2 || dense.lgElmtOn[nelem] )
			{
				/* note that pesc and dest are updated no matter what escprob logic we
				 * specify, and that these are not updated if we have overrun the
				 * optical depth scale.  only update here in that case */
				if( lgTauGood( &Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0] ) )
				{
					if( lgDoEsc )
					{
						for( ipHi=1; ipHi < iso.numLevels_max[ipISO][nelem]; ipHi++ )
						{
							for( ipLo=0; ipLo < ipHi; ipLo++ )
							{
								if( Transitions[ipISO][nelem][ipHi][ipLo].ipCont <= 0 )
									continue;

								/* >>chng 03 jun 07, do not clobber esp prob when line is masing -
								 * this had effect of preventing total escape prob from getting larger than 1 */
								if( Transitions[ipISO][nelem][ipHi][ipLo].Emis->Pesc<1. )
								{
									Transitions[ipISO][nelem][ipHi][ipLo].Emis->Pesc = MIN2(1.f,
										Transitions[ipISO][nelem][ipHi][ipLo].Emis->Pesc+
										(realnum)iso.pestrk[ipISO][nelem][ipLo][ipHi]);
								}
							}
						}
					}
					/* always add on stark broadening term to Lya*/
					else if( Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pesc < 1. )
					{
						Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pesc = MIN2(1.f,
							Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pesc+
							(realnum)iso.pestrk[ipISO][nelem][0][iso.nLyaLevel[ipISO]]);
					}
				}
			}
		}
	}

	/* note that pesc and dest are updated no matter what escprob logic we
	 * specify, and that these are not updated if we have overrun the
	 * optical depth scale.  only update here in that case */
	if( lgTauGood( &Transitions[ipH_LIKE][ipHYDROGEN][iso.nLyaLevel[ipH_LIKE]][0] ) )
	{
		/* do the 8446 problem */
		atom_oi_calc(&coloi);

		Transitions[ipH_LIKE][ipHYDROGEN][ipH3p][ipH1s].Emis->Pesc = (realnum)(atoms.pmph31/
			Transitions[ipH_LIKE][ipHYDROGEN][ipH3p][ipH1s].Emis->Aul);

		if( trace.lgTrace && trace.lgIsoTraceFull[ipH_LIKE] )
		{
			fprintf( ioQQQ, "       RT_line_all calls P8446 who found pmph31=%10.2e\n", 
				atoms.pmph31 );
		}

		/* add on destruction of hydrogen Lya by FeII
		* now add in FeII deexcitation via overlap,
		* but only as loss, not photoionization, source
		* dstfe2lya is Ly-alpha deexcit by FeII overlap - conversion into FeII em */
		/* find FeII overlap destruction rate, 
		* this does NOTHING when large FeII atom is turned on, 
		* in this case evaluation is done in call to FeIILevelPops */
		t_fe2ovr_la::Inst().atoms_fe2ovr();
		/*fprintf(ioQQQ,"DEBUG fe2 %.2e %.2e\n", hydro.dstfe2lya ,
			hydro.HLineWidth);*/

		/* >>chng 00 jan 06, let dest be large than 1 to desaturate the atom */
		/* >>chng 01 apr 01, add test for tout >= 0., 
		 * must not add to Pdest when it has not been refreshed here */
		Transitions[ipH_LIKE][ipHYDROGEN][ipH2p][ipH1s].Emis->Pdest += hydro.dstfe2lya;
	}

	/* take mean of old and new to damp oscillations in Lya (only) */
	if( nzone > 1 )
	{
		for( ipISO=ipH_LIKE; ipISO<NISO; ++ipISO )
		{
			/* loop over all iso-electronic sequences */
			for( nelem=ipISO; nelem<LIMELM; ++nelem )
			{
				ion = nelem+1-ipISO;
				if( ion <=dense.IonHigh[nelem] && ipISO<=nelem )
				{
					/* now take mean of old and new escape dest probs for Lya */
					/* only ipLY_A redist did not already take average */
					if( Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->iRedisFun==ipLY_A &&
						lgTauGood( &Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0] ) )
					{
						/* the Lya routine is different in that escape and dest
						 * probs are always updated, unless we have overrun the
						 * optical depth scale */
						/*lint -e644 SaveLyaPdest not init */
						Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pdest = 
							((realnum)SaveLyaPdest[ipISO][nelem] + 
							Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pdest) / 2.f;
						Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pesc = 
							((realnum)SaveLyaPesc[ipISO][nelem] + 
							Transitions[ipISO][nelem][iso.nLyaLevel[ipISO]][0].Emis->Pesc) / 2.f;
						/*lint +e644 SaveLyaPdest not init */
					}
				}
			}
		}
	}
	/*5986 fprintf(ioQQQ,"DEBUG he rt_all\t%li\t%li\t%.4e\t%.4e\n",
		iteration, nzone, 
				Transitions[1][1][10][5].Emis->TauIn,
				Transitions[1][1][10][5].Emis->Pesc);*/

	/* is continuum pumping of H Lyman lines included?  yes, but turned off
	 * with atom h-like Lyman pumping off command */
	if( !hydro.lgLymanPumping )
	{
		ipISO = ipH_LIKE;
		nelem = ipHYDROGEN;
		ipLo = 0;
		for( ipHi=ipLo+1; ipHi < iso.numLevels_max[ipISO][nelem]; ++ipHi )
		{
			/* negative ipCont means this is not a real line, so do not
				* transfer it */
			Transitions[ipISO][nelem][ipHi][ipLo].Emis->pump = 0.;
		}
	}

	{
		/* following should be set true to print ots contributors for he-like lines*/
		/*@-redef@*/
		enum {DEBUG_LOC=false};
		/*@+redef@*/
		if( DEBUG_LOC && nzone>433 /*&& iteration==2*/ )
		{
			/* option to dump a line  */
			DumpLine(&Transitions[ipH_LIKE][ipHYDROGEN][ipH2p][0] );
		}
	}

	/* level 1 lines */
	for( i=1; i <= nLevel1; i++ )
	{
		RT_line_one(&TauLines[i] , lgDoEsc , lgUpdateFineOpac,true);
	}
	/* co carbon monoxide */
	for( i=0; i < nCORotate; i++ )
	{
		RT_line_one(&C12O16Rotate[i] , lgDoEsc , lgUpdateFineOpac,true);
		RT_line_one(&C13O16Rotate[i] , lgDoEsc , lgUpdateFineOpac,true);
	}
	for( i=0; i < nHFLines; i++ )
	{
		RT_line_one(&HFLines[i] , lgDoEsc , lgUpdateFineOpac,true);
	}
	/* Database Atoms & Molecules*/
	for( i=0; i < linesAdded2; i++)
	{
		RT_line_one(atmolEmis[i].tran , lgDoEsc , lgUpdateFineOpac,true);
	}
	/* this is a major time sink for this routine */
	if( lgRT_update )
	{
		for( i=0; i < nUTA; i++ )
		{
			/* Aul = -1 means this line is superceded by better calculations in
			 * different data set */
			if( UTALines[i].Emis->Aul > 0. )
			{
				/* these are not defined in cooling routines so must be set here */
				UTALines[i].Emis->PopOpc = dense.xIonDense[UTALines[i].Hi->nelem-1][UTALines[i].Hi->IonStg-1];
				UTALines[i].Lo->Pop = dense.xIonDense[UTALines[i].Hi->nelem-1][UTALines[i].Hi->IonStg-1];
				UTALines[i].Hi->Pop = 0.;
				RT_line_one(&UTALines[i] , lgDoEsc , lgUpdateFineOpac,true);
			}
		}
	}

	/* h-like ions only have one electron, no satellite lines. */
	for( ipISO=ipHE_LIKE; ipISO<NISO; ++ipISO )
	{
		/* loop over all iso-electronic sequences */
		for( nelem=ipISO; nelem<LIMELM; ++nelem )
		{
			if( dense.lgElmtOn[nelem] )
			{
				for( ipLo=1; ipLo < (iso.numLevels_local[ipISO][nelem]-1); ++ipLo )
				{
					RT_line_one(&SatelliteLines[ipISO][nelem][ipLo], lgDoEsc, lgUpdateFineOpac, true);
				}
			}
		}
	}

	/* >>chng 03 aug 22, must call RT_line_one every single time, since that routine
	 * establishes the fine opacities for the level 2 lines */
	/* >>chng 01 aug 11, the level 2 lines were only updated when lgDoEsc was true,
	 * this meant that dest probs were not updated but once, causing very high-Z models
	 * to develop numerical oscialltions.  removed if, now evaluate level 2 always */
	/* only update their dest/esc probs one time per zone */
	/* lgLevel2_OTS_Imp set true in dimacool if ots rates were significant */
	/* level 2 heavy element lines in cooling with only g-bar,*/
	for( i=0; i < nWindLine; i++ )
	{
		/* >>chng 02 sug 11, do not include ions done in iso-sequences */
		/*if( TauLine2[i].IonStg <= TauLine2[i].nelem-1 )*/
		if( TauLine2[i].Hi->IonStg < TauLine2[i].Hi->nelem+1-NISO )
		{
			RT_line_one(&TauLine2[i] , lgDoEsc , lgUpdateFineOpac,true);
		}
	}

	/* the large H2 molecule */
	H2_RTMake( lgDoEsc , lgUpdateFineOpac);

	/* The large model FeII atom */
	FeII_RT_Make( lgDoEsc , lgUpdateFineOpac);
	return;
}
