/* This file is part of Cloudy and is copyright (C)1978-2008 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */

#ifndef _WIND_H_
#define _WIND_H_

/** parameters for wind model; wind.h */
EXTERN struct t_wind {

	/** initial wind velocity cm s-1, will be negative for D-critical flow */
	realnum windv0;

	/** central object mass in solar units, to compute inward acceleration */
	realnum comass;

	/** current wind velocity cm s-1, code tests for wind solution by
	 * seeing whether this is positive, static solution if 0 */
	realnum windv;

	/** mass flux at inner radius, n*v [cm-3 cm s-1 == cm-2 s-1], 
	 * set in parsecommands at inner boundary.  needs r^2 to be real mass flux 
	 * density set by conservation of this quantity in wind model */
	realnum emdot;

	/** *flag set if wind not ok, (below sonic point) */
	bool lgWindOK;

	/** variables for tracking average radiative acceleration - used to report
	 * average acceleration at end of calculation */
	realnum AccelAver, acldr;

	/** inward gravitational acceleration, cm s-2, a positive number */
	realnum AccelGravity;

	/**AccelTot is total radiative acceleration, lines and continuum, cm s-2 */
	realnum AccelTot;

	/**AccelCont continuum radiative acceleration - cm s-2 */
	realnum AccelCont;

	/**AccelLine line radiative acceleration - cm s-2 */
	realnum AccelLine;

	/**AccelPres change in pressure acceleration - cm s-2 */
	realnum AccelPres;

	/** force multiplier - dimensionless */
	realnum fmul;

	/**AccelMax is largest acceleration, usually at illuminated face - cm s-2 */
	realnum AccelMax;

	/** flag set if wind solution had negative velocity */
	bool lgVelPos;

	/* option to do rotating disk, set true with DISK option on wind command */
	bool lgDisk;

	/* for rotating disk this is inner radius, else it is zero,
	 * divide by this to get centrifugal acceleration */
	double DiskRadius;

	}	wind;


#endif /* _WIND_H_ */
